<!doctype html>
<html>
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <title>{% block title %}{% endblock %}</title>
        <meta name="viewport" content="width=device-width, initial-scale=1">

        {% block css %}
        {% endblock %}
    </head>
    <body>
    {% block content %}
    {% endblock %}
    </body>
</html>